// mpolice.h - memory policing
// Owner:  Nikolaus Seydel, Amadeus Data Processing
// Date:    19 Jan 2009
// Summary: mpolice (memory police) can be used to detect strange memory
//          (heap) usage, eg. leaking heap, functional heap leakage, etc.
//          mpolice comes with an executable (mpolice) and with a shared
//          library (libmpolice).
// See mpolice.c for more details

#ifndef _MPOLICE_H_
#define _MPOLICE_H_

#include "mpconfig.h"
#include <unistd.h>
#include <limits.h>
#include <ctype.h>
#include <time.h>
#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <stdint.h>

// package common defines

#define USE_SIGNALS    0         // was 1 (to use signal support)
#define SIG_DEFAULT    SIGUSR1   // default action (status or delta)
#define SIG_COMMAND    SIGUSR2   // action read from temp file

// some definitions

#ifdef __gnu_linux__
#  define PLATFORM "GNU Linux"
#else
#  define PLATFORM "Linux"
#endif

#define SIG_FILE "/tmp/mpolice.%d"
#define LOG_NEW_NAME "%s%d"
#define INIT_MUTEX(m)
#define FINI_MUTEX(m)

#if __WORDSIZE==32
#   define PTRFMT "%08lx"
#else
#   define PTRFMT "%016lx"
#endif

// heap chunk prefix contains chunk size
// Linux/32 chunks have a prefix of 1 long: size + bits (007)
// Linux/64 chunks have the same prefix, but with 64-bit words
// When using long, this turns out to be identical (int would fail)

#define MSIZE(p) (((long *) (p))[-1] & -8)

// function forwarding setup

#define mALLOc         malloc
#define cALLOc         calloc
#define rEALLOc        realloc
#define fREe           free
#define fORk           fork

#define __initattr     __attribute__((constructor))
#define __finisattr    __attribute__((destructor))
#define __mpextern     static
#define __main         main

extern void * __libc_malloc(size_t);
extern void * __libc_calloc(size_t, size_t);
extern void * __libc_realloc(void *, size_t);
extern void   __libc_free(void *);
extern pid_t  __libc_fork(void);

#ifdef WINDOWSNT
#   include "mpwoe.h" // additional windows stuff
#endif

#endif // #ifndef _MPOLICE_H_
